unit ObjectInspec;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, Grids, ExtCtrls,TypInfo,DsgnIntf, Buttons, Menus;

type
  TObjectInspector = class(TForm)
    PageControl1: TPageControl;
    TabProperty: TTabSheet;
    TabEvent: TTabSheet;
    Panel1: TPanel;
    ComponentList: TComboBox;
    FontDialog1: TFontDialog;
    StatusBar1: TStatusBar;
    PropertyBox: TListBox;
    EventBox: TListBox;
    cbValue: TComboBox;
    edProperty: TEdit;
    Browse: TPanel;
    edEvent: TEdit;
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure ComboValueChange(Sender: TObject);
    procedure StringGrid2DblClick(Sender: TObject);
    procedure PropertyBoxDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure PropertyBoxMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PropertyBoxMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure PropertyBoxMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormShow(Sender: TObject);
    procedure EventBoxDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure EventBoxMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure EventBoxMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure EventBoxMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PropertyBoxClick(Sender: TObject);
    procedure PropertyBoxKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure EventBoxClick(Sender: TObject);
    procedure cbValueDblClick(Sender: TObject);
    procedure BrowseClick(Sender: TObject);
    procedure edPropertyKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure edPropertyKeyPress(Sender: TObject; var Key: Char);
    procedure cbValueChange(Sender: TObject);
    procedure cbValueKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure cbValueDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure FormActivate(Sender: TObject);
    procedure ComponentListClick(Sender: TObject);
    procedure edEventDblClick(Sender: TObject);
    procedure PropertyBoxDblClick(Sender: TObject);
  private
    { Private declarations }
    EventCount:integer;
    fComponent: TComponent;
    fPropertyName : string;
    fPropertyAttr: TPropertyAttributes;
    PropertyList: TStringList;
    EventList: TStringList;
    SubPos:Integer;
    HalfWay: Integer;
    EditRect: TRect;
    SplitDrag: Boolean;
    fPropertyEditor: TPropertyEditor;
    function  ExtractComponentName(const S:string):string;
    procedure ClearProperties;
    procedure GetProperties(Component: TComponent);
    procedure GetPropEditor(Editor: TPropertyEditor);
    procedure GetEventEditor(Editor: TPropertyEditor);
    procedure SetEditor(Editor: TPropertyEditor);
    function  GetComponentName: string;
    function  GetComponent(var Wrapper:TComponent): TComponent;
    procedure EditorProperty;
    procedure PropSetValue(const Value: string);
    procedure UpdateValue;
    function  GetPropValue(Info:PPropInfo):TStringList;
    procedure InsertPropInfo(Info:PPropInfo;Sender:TObject);
    procedure DeletePropInfo(Info:PPropInfo;Sender:TObject);
    procedure SetEditBounds(AControl : TWinControl);
    procedure GetValueProc(const Value: string);
    procedure GetSubPropEditor(Editor: TPropertyEditor);
    function  GetPropertyName: string;
    function  GetEventName: string;
    function  GetPropertyValueName: string;
    function  GetEventValueName: string;
    procedure SetPropertyValueName(Value:string) ;
    procedure SetcbValueIndex(Text:string);
  protected
    procedure Initialize(Component: TComponent);
  public
    { Public declarations }
     constructor CreateMain(Owner: TComponent; Component: TComponent);
     function  WritePropInfo(Info:PPropInfo;I:integer;Sender:TObject):string;
     procedure WritePropertyInfo(Info:PTypeInfo;Data:PTypeData;Sender:TObject);
     procedure GetPropertyInfo(Sender: TObject);
     procedure DisplayProperty(Wrapper,Component:TComponent);
     procedure InitializeInspector;
     property  PropertyEditor: TPropertyEditor read fPropertyEditor;
     property  PropertyName: string read fPropertyName write fPropertyName;
     property  SelectComponent: TComponent read fComponent write fComponent;
     property  PropAttrs: TPropertyAttributes read fPropertyAttr;
 end;

var
  ObjectInspector: TObjectInspector;

implementation

uses Proxy, MainForm, utype, Editor,URegister,uconst;

type
   TIntegerSet=set of 0..Sizeof(Integer)*8-1;

{$R *.DFM}

  TParamFlag = (pfVar, pfConst, pfArray);
  TParamFlags = set of TParamFlag;
  PParamString = PShortString;
  TParamInfo = record
    Flags: TParamFlags;
    ParamName: PParamString;
    ParamType: PParamString;
  end;
  TParamList = array[0..255] of TParamInfo;
  PParamList = ^TParamList;


{ Like 'is,' but works with a string. }
function IsA(PI : PTypeInfo; S : String) : Boolean;
Var PTD : PTypeData;
    PTI : PTypeInfo;
    Found : Boolean;
Begin
  Found := False;
  S := UpperCase(S);
  PTI := PI;
  While (Not Found) and (UpperCase(PTI^.Name) <> 'TOBJECT') do Begin
    Found := UpperCase(PTI^.Name) = S;
    PTD := GetTypeData(PTI);
    PTI := PTD^.ParentInfo^;
  End;
  IsA := Found;
End;

{ Get the infromation for a parameter list, and store
  it in the ParamList argument. }
procedure GetParamInfo(Data: PTypeData; var Params: TParamList;
                       var ReturnType: PParamString);
var
  I: Integer;
  Ptr: PByte;
begin
  with Data^ do
  begin
    Ptr := PByte(@ParamList);
    for I := 0 to ParamCount-1 do
      with Params[I] do
      begin
        Flags := TParamFlags(Ptr^);
        Inc(Ptr);
        ParamName := PParamString(Ptr);
        Inc(Ptr, Length(ParamName^) + 1);
        ParamType := PParamString(Ptr);
        Inc(Ptr, Length(ParamType^) + 1);
      end;
    if MethodKind = mkFunction then
      ReturnType := PParamString(Ptr);
  end;
end;

{ Write the information for a method, including all the parameters.
  If the method is a function, then show the return type, too. }

function WriteMethodData(Data: PTypeData):string;
var
  I: Integer;
  Params: PParamList;
  Return: PParamString;
  MethodPara:string;
begin
  with Data^ do
  begin
    { Allocate memory to hold all the parameter information }
    GetMem(Params, ParamCount * SizeOf(TParamInfo));
    try
      GetParamInfo(Data, Params^, Return);
      { Write each parameter: }
      for I := 0 to ParamCount-1 do
        with Params^[I] do
        begin
          if pfVar in Flags then
            MethodPara:=MethodPara+'var ';
          if pfConst in Flags then
            MethodPara:=MethodPara+'const ';
          MethodPara:=MethodPara+ParamName^+ ': ';
          if pfArray in Flags then
            MethodPara:=MethodPara+'array of ';
          MethodPara:=MethodPara+ParamType^+';';
        end;
    finally
      FreeMem(Params, ParamCount * SizeOf(TParamInfo));
    end;
  end;
  if MethodPara[Length(MethodPara)]=';' then
     MethodPara:=Copy(MethodPara,1,Length(MethodPara)-1);
  Result:=MethodPara;
end;

constructor TObjectInspector.CreateMain(Owner: TComponent; Component: TComponent);
begin
  inherited Create(Owner);
  Initialize(Component);
  GetProperties(Component);
end;

// Ư Ӽ Ǵ ̺Ʈ   ش.
function TObjectInspector.WritePropInfo(Info:PPropInfo;I:integer;Sender:TObject):string;
var
   PropValue:string;
   OrdValue,MinVal,MaxVal:integer;
   Data:PTypeData;
   S:TIntegerSet;
   J:Integer;
   TypeInfo1:PTypeInfo;
   MethodValue: TMethod;
begin
    Result:= '';
    if Sender = nil then Exit;
    if Info^.PropType^.Kind=tkMethod then begin
       MethodValue:= GetMethodProp(Sender,Info);
     //  PropValue:=(Designer as TProxyDesigner).GetMethodName(MethodValue);
       PropValue:= '';
       Inc(EventCount,1);
    end
    else begin
      case Info^.PropType^.Kind of
          tkString,tkLString:  // Ӽ ڿ 
             PropValue:=GetStrProp(Sender,Info);
          tkEnumeration: begin
              OrdValue:=GetOrdProp(Sender,Info);
              if Info.Name='BOOLEAN' then begin
                  if OrdValue=0 then PropValue:='False'
                  else PropValue:='True'
              end
              else begin
                  PropValue:=GetEnumName(Info.PropType^,OrdValue);
              end;
          end;
          tkClass: begin
              Data:=GetTypeData(Info^.PropType^);
              if Data^.PropCount>0 then
              PropValue:='('+Data^.ClassType.ClassName+')';
          end;
          tkInteger: begin
              OrdValue:=GetOrdProp(Sender,Info);
              PropValue:=UpperCase(Info.Name);
              if Pos('COLOR',PropValue)>0 then // Ӽ Color 
                 PropValue:=ColorToString(TColor(OrdValue))
              else if Pos('CURSOR',PropValue)>0 then // Ӽ cursor 
                 PropValue:=CursorToString(TCursor(OrdValue))
              else PropValue:=IntToStr(OrdValue);
          end;
          tkSet: begin
              Data:=GetTypeData(Info^.PropType^);
              TypeInfo1:=Data^.CompType^;
              Data:=GetTypeData(TypeInfo1);
              MinVal:=Data^.MinValue;
              MaxVal:=Data^.MaxValue;
              Integer(S):=GetOrdProp(Sender,Info);
              PropValue:='[';
              for J:=MinVal to MaxVal do
                if I in S then begin
                    if Length(PropValue)<>1 then PropValue:=PropValue+',';
                    PropValue:=PropValue+GetEnumName(TypeInfo1,J);
                 end;
              PropValue:=PropValue+']';
          end;
       end;
    end;
    Result:= PropValue;
end;

//  Ӽ  Ӽ  Ӽ ش.
procedure TObjectInspector.InsertPropInfo(Info:PPropInfo;Sender:TObject);
var
   PropName, PropValue, Prop:string;
   Data:PTypeData;
   I, Count:Integer;
   TypeInfo:PTypeInfo;
   PropList:PPropList;
begin
   if Info = nil then Exit;
   if Sender<>nil then begin
      TypeInfo:= Info^.PropType^;
      Count:=GetPropList(TypeInfo,tkProperties,nil);
      GetMem(PropList, Count*SizeOf(PPropInfo));
      GetPropInfos(TypeInfo, PropList);
      case Info^.PropType^.Kind of
          tkClass: begin
              Data:=GetTypeData(Info^.PropType^);
              for I:=0 to Data^.PropCount - 1 do begin
                 PropName:= '  ' + PropList^[I].Name;
                // PropValue:= WritePropInfo(PropList^[I], 0, Sender);
                 Prop:= Format('%-20s%-20s',[PropName, PropValue]);
                 PropertyBox.Items.Insert(PropertyBox.ItemIndex + 1, Prop);
              end;
          end;
      end;
      FreeMem(PropList, Count*SizeOf(PPropInfo));
   end;
end;

//  Ӽ  Ӽ  Ӽ .
procedure TObjectInspector.DeletePropInfo(Info:PPropInfo;Sender:TObject);
var
   Data:PTypeData;
   I,MinVal,MaxVal:Integer;
   TypeInfo:PTypeInfo;
begin
   if Info = nil then Exit;
   if Sender<>nil then begin
      case Info^.PropType^.Kind of
          tkClass: begin
              Data:=GetTypeData(Info^.PropType^);
              for I:=Data^.PropCount - 1 downto 0 do begin
                 PropertyList.Delete(PropertyBox.ItemIndex + I + 1);
                 PropertyBox.Items.Delete(PropertyBox.ItemIndex + I + 1);
              end;
          end;
          tkSet: begin
              Data:=GetTypeData(Info^.PropType^);
              TypeInfo:=Data^.CompType^;
              Data:=GetTypeData(TypeInfo);
              MinVal:=Data^.MinValue;
              MaxVal:=Data^.MaxValue;
              for I:=MaxVal-MinVal  downto 0 do begin
                 PropertyList.Delete(PropertyBox.ItemIndex + I + 1);
                 PropertyBox.Items.Delete(PropertyBox.ItemIndex + I + 1);
              end;
          end;
      end;
   end;
end;

// Ư ŸԿ  Ӽ ̺Ʈ  Ѵ.
procedure TObjectInspector.WritePropertyInfo(Info:PTypeInfo;Data:PTypeData;Sender:TObject);
var
   I,Count,Count1:integer;
   PropList,PropList1:PPropList;
   Prop:string;

procedure AddPropItem(ListBox:TListBox;Info:PPropInfo);
begin
    if Info^.PropType^.Kind in [tkClass] then begin
       if IsA(Info^.PropType^, 'TCOMPONENT') then
          ListBox.Items.Add(Format('%-20s',[Info^.Name])+ Prop)
       else ListBox.Items.Add(Format('+%-20s',[Info^.Name])+ Prop);
    end
    else if Info^.PropType^.Kind in [tkSet] then
       ListBox.Items.Add(Format('+%-20s',[Info^.Name])+ Prop)
    else ListBox.Items.Add(Format('%-20s',[Info^.Name])+ Prop);
end;
begin
    EventCount:=0;
    Count:=GetPropList(Info,tkProperties,nil);   // ̺Ʈ Ӽ 
    Count1:=GetPropList(Info,tkMethods,nil);   // ̺Ʈ Ӽ 
    GetMem(PropList,Count*SizeOf(PPropInfo));
    GetMem(PropList1,Count1*SizeOf(PPropInfo));
    try
       GetPropList(Info,tkProperties,PropList);
       for I:=0 to Count-1 do begin
          Prop:= WritePropInfo(PropList^[I],I,Sender);
          AddPropItem(PropertyBox, PropList^[I]);
       end;
       GetPropList(Info,tkMethodS,PropList1);
       for I:=0 to Count1-1 do begin
          Prop:=WritePropInfo(PropList1^[I],I,Sender);
          AddPropItem(EventBox, PropList1^[I]);
       end;
    finally
       FreeMem(PropList,Count*SizeOf(PPropInfo));
       FreeMem(PropList1,Count1*SizeOf(PPropInfo));
    end;
end;

procedure TObjectInspector.ClearProperties;
begin
   PropertyBox.Clear;
   EventBox.Clear;
   PropertyList.Clear;
end;

// ۳Ʈ  Ӽ ̺Ʈ  ׸   ´.
procedure TObjectInspector.GetPropertyInfo(Sender: TObject);
var
  Info:PTypeInfo;
  Data:PTypeData;
begin
   Info:=Sender.ClassInfo;
   Data:=GetTypeData(Info);
   GetProperties(Sender as TComponent);
   WritePropertyInfo(Info,Data,Sender);
end;

{ The user has chosen a property, so get the editor,
  and initialize the property editor fields. }
procedure TObjectInspector.SetEditor(Editor: TPropertyEditor);
begin
   fPropertyEditor:=Editor;
   if Editor= nil then  Exit;
   Editor.Activate;
   fPropertyAttr:=Editor.GetAttributes;
   if (paDialog in PropAttrs) then SetEditBounds(Browse)
   else SetEditBounds(nil);
   cbValue.Clear;
   //Ӽ ̺Ʈ ƴϰ    Ӽ  ش.
   if ((paValueList in PropAttrs) and (Editor.ClassName<>'TMethodProperty')) then begin
      Editor.GetValues(GetValueProc);
      SetEditBounds(cbValue);
   end
   else if Editor.ClassName='TFontProperty' then SetEditBounds(Browse)
   else if Editor.ClassName='TPictureProperty' then SetEditBounds(Browse)
   else if Editor.ClassName='THD_PictureProperty' then SetEditBounds(Browse)
   else if Editor.ClassName='THD_StringListProperty' then SetEditBounds(Browse)
   else if (Editor.ClassName ='TMethodProperty') then SetEditBounds(edEvent)
   else if  (Editor.ClassName ='THD_MenuItemProperty') then  SetEditBounds(Browse)
   else if  (Editor.GetPropType.Name='TBitmap') then  SetEditBounds(Browse)
   else if  (Editor.GetPropType.Name='TPopupMenu') then  SetEditBounds(cbValue)
   else if  (Editor.GetPropType.Name='TListColumns') then  SetEditBounds(Browse);
  // else  SetEditBounds(edProperty);
   StatusBar1.SimpleText:=Editor.GetPropType.Name+' : '+Editor.ClassName;
   // Ӽ ̺Ʈ ƴϸ Ӽ Ѵ.
   if (Editor.ClassName<>'TMethodProperty') then  UpdateValue;
  // edProperty.Text:=GetPropertyValueName;
   if edProperty.Visible then  edProperty.Text:= Editor.GetValue
   else if cbValue.Visible then  SetcbValueIndex(Editor.GetValue);
end;

{ Callback for getting a subproperty editor. }
procedure TObjectInspector.GetPropEditor(Editor: TPropertyEditor);
begin
   Editor.Initialize;
   PropertyList.AddObject(Editor.GetName, Editor);
end;

// Font  SubProperties  Ӽ  Ӽ
// ִ Callback ν
procedure TObjectInspector.GetSubPropEditor(Editor: TPropertyEditor);
var
  Prop:string;
begin
   Editor.Initialize;
   Prop:= Format('  %-18s%-s',[Editor.GetName, Editor.GetValue]);
   PropertyList.InsertObject(PropertyBox.ItemIndex + 1, Editor.GetName, Editor);
   PropertyBox.Items.Insert(PropertyBox.ItemIndex + 1, Prop);
end;

procedure TObjectInspector.GetEventEditor(Editor: TPropertyEditor);
begin
   EventList.AddObject(Editor.GetName, Editor);
end;

// ۳Ʈ  ִ Ӽ   ش.
procedure TObjectInspector.GetProperties(Component: TComponent);
var
  CompList: TComponentList;    // Ӽ    
  Comp1List: TComponentList;   // ̺Ʈ  
begin
  PropertyList.BeginUpdate;
  EventList.BeginUpdate;
  ClearProperties;
  CompList := TComponentList.Create;
  Comp1List := TComponentList.Create;
  try
    CompList.Add(Component);
    Comp1List.Add(Component);
    // ۳ƮComponent  Ӽ ⸦ Ѵ.(Ӽ  )
    GetComponentProperties(CompList, tkProperties,ProxyDesigner , GetPropEditor);
    // ۳Ʈ  Ӽ(̺Ʈ)  Ӽ ⸦ Ѵ.( ̺Ʈ)
    GetComponentProperties(Comp1List, tkMethods, ProxyDesigner, GetEventEditor);
  finally
    CompList.Free;
    Comp1List.Free;
  end;
  PropertyList.EndUpdate;
  EventList.EndUpdate;
end;

{ Initialize the property editor form. }
procedure TObjectInspector.Initialize(Component: TComponent);
begin
  Designer := ProxyDesigner;
  fComponent := Component;
end;

procedure TObjectInspector.FormCreate(Sender: TObject);
begin
   URegister.Register;   //  Ӽ ⸦ Ѵ.
   PropertyList:=TStringList.Create;
   EventList:=TStringList.Create;
   Left:=0;
   PageControl1.ActivePage:=TabProperty;
   ComponentList.Items.Add('Form1:TForm1');
end;

procedure TObjectInspector.FormResize(Sender: TObject);
begin
   //  ũⰡ ٲ   Ʈ ̸ Ѵ.
   ComponentList.Width:=Width-10;
   EditRect.Right := PropertyBox.ItemRect(PropertyBox.ItemIndex).Right;
   if Browse.Visible then SetEditBounds(Browse)
   else if edProperty.Visible then SetEditBounds(edProperty)
   else if edEvent.Visible then SetEditBounds(edEvent)
   else if cbValue.Visible then SetEditBounds(cbValue);
end;

// Ʈ νƮ ۳Ʈ Ͽ ۳Ʈ ̸ Ѵ.
function TObjectInspector.ExtractComponentName(const S:string):string;
var
  SepaPos:integer;
begin
   Result:='';
   SepaPos:=Pos(':',S);
   if SepaPos>1 then Result:=Copy(S,1,SepaPos-1);
end;

//   ۳Ʈ ̸ ش.
function TObjectInspector.GetComponentName: string;
begin
  with ComponentList do
    if ItemIndex<0 then   Result := ''
    else  Result := ExtractComponentName(Items[ItemIndex]);
end;

// Object Inspector  ۳Ʈ ش.
// ð ۳Ʈ  쿡 Wrapper WrapperControl ҴѴ.
function TObjectInspector.GetComponent(var Wrapper:TComponent): TComponent;
var
  WrapperName,ComponentName:string;
  Component:TComponent;
begin
  Wrapper:=nil;
  ComponentName:= GetComponentName;
  WrapperName:='Wrapper'+ComponentName;
  if ComponentName = ProxyDesigner.Form.Name then
    Result := ProxyDesigner.Form
  else begin
    Component:= ProxyDesigner.Form.FindComponent(WrapperName);
    if Component<>nil then  Wrapper:=Component;
    Result := ProxyDesigner.Form.FindComponent(ComponentName);
  end;
end;

// ۳Ʈ  Ʈ νƮ  ش.
// ð ۳Ʈ 쿡 Ӽ ð ۳Ʈ ְ, 
// ð ۳Ʈ Wrapper۳Ʈ Ѵ.
// ð ۳Ʈ 쿡 Wrapper nil ƴ
procedure TObjectInspector.DisplayProperty(Wrapper,Component:TComponent);
var
  Name:string;
begin
   if Component<>nil then begin
      SetEditBounds(nil);   //  Ʈ  ʰ Ѵ.
      GetPropertyInfo(TObject(Component));
      if Wrapper<>nil then   ProxyDesigner.SelectComponent(Wrapper)
      else ProxyDesigner.SelectComponent(Component);
      if Component is TForm then Name:= 'TForm'
      else Name:= Component.ClassName;
      SelectComponent:= Component;
      ComponentList.ItemIndex:= ComponentList.Items.IndexOf(Component.Name+':'+Name);
   end;
end;

procedure  TObjectInspector.EditorProperty;
begin
   if (PropertyEditor <> nil) then begin
       PropertyEditor.Edit;
       UpdateValue;
   end;
   SetPropertyValueName(PropertyEditor.GetValue);
end;

// Ӽ  Value Ѵ.
procedure TObjectInspector.PropSetValue(const Value: string);
begin
   if PropertyEditor <> nil then begin
      PropertyEditor.SetValue(Value);
      UpdateValue;
   end;
end;

procedure TObjectInspector.UpdateValue;
begin
  // if PropertyEditor<> nil then
  //     SetPropertyValueName(PropertyEditor.GetValue);
end;

// Ӽ   Ǵ  ش.(ڿ)
function TObjectInspector.GetPropValue(Info:PPropInfo):TStringList;
var
   Data:PTypeData;
   I,MinVal,MaxVal: Integer;
   ValueList: TStringList;
   TypeInfo1 :PTypeInfo;
begin
   ValueList:=TStringList.Create;
   Data:= GetTypeData(Info^.PropType^);
   case Info^.PropType^.Kind of
        tkEnumeration: begin
           for I:=Data.MinValue to Data.MaxValue do
               ValueList.Add(GetEnumName(Info^.PropType^,I));
           end;
           tkSet: begin
               Data:=GetTypeData(Info^.PropType^);
               TypeInfo1:=Data^.CompType^;
               Data:=GetTypeData(TypeInfo1);
               MinVal:=Data^.MinValue;
               MaxVal:=Data^.MaxValue;
               for I:=MinVal to MaxVal do
                  ValueList.Add(GetEnumName(TypeInfo1,I));
            end;
   end;
   Result:= ValueList;
end;

// Ӽ type  Ӽ Է Ʈ Ѵ.
procedure TObjectInspector.SetEditBounds(AControl : TWinControl);
var
  Index: Integer;
begin
  // Ӽ  Ǵ  , Ӽ ޺ ڽ Ӽ ġ ߰,
  // ׷  ޺ ڽ  ʰ Ѵ.
  if AControl<>nil then with AControl do
  begin
    if AControl=cbValue then begin
      cbValue.SetBounds(EditRect.Left+1,EditRect.Top,
                 EditRect.Right-EditRect.Left+3,EditRect.Bottom - EditRect.Top);
      cbValue.Visible:=True;
      Browse.Visible:=False;
      edProperty.Visible:= False;
      cbValue.SetFocus;
      Index:= cbValue.Items.IndexOf(GetPropertyValueName);
      if Index<>-1 then cbValue.ItemIndex:= Index
      else cbValue.Text:= GetPropertyValueName;
    end
    else if AControl=edProperty then begin
      edProperty.SetBounds(EditRect.Left+1,EditRect.Top,
                 EditRect.Right-EditRect.Left+3,EditRect.Bottom - EditRect.Top);
      cbValue.Visible:=False;
      Browse.Visible:=False;
      edProperty.Visible:= True;
     // edProperty.SetFocus;
    end
    else if AControl=Browse then begin
      Browse.SetBounds(EditRect.Right-15,EditRect.Top,Browse.Width,EditRect.Bottom - EditRect.Top);
      Visible:=True;
      Browse.SetFocus;
      cbValue.Visible:=False;
      edProperty.Visible:= False;
    end
    else if AControl=edEvent then begin
      edEvent.SetBounds(EditRect.Left+1,EditRect.Top,
                 EditRect.Right-EditRect.Left+3,EditRect.Bottom - EditRect.Top);
      edEvent.Width:= EventBox.ClientWidth - HalfWay;
      edEvent.Text:= Copy(EventBox.Items[EventBox.ItemIndex], 21, 255);
      Visible:=True;
      edEvent.SetFocus;
    end;
  end
  else begin
     cbValue.Visible:=False;
     Browse.Visible:=False;
     edProperty.Visible:= False;
  end;
end;

procedure TObjectInspector.ComboValueChange(Sender: TObject);
begin
   PropSetValue(cbValue.Text);
end;

// ̺Ʈ  ҽ ڵ带 Ѵ.
procedure TObjectInspector.StringGrid2DblClick(Sender: TObject);
var
   EventName: string;
   EventHandler:string;
begin
    EventName:=ExtractComponentName(ComponentList.text);
   //// with StringGrid2 do
   ////    EventName:=EventName+Copy(Cells[0,Row],3,Length(Cells[0,Row]));
    EventHandler:=EventName;
    // ̺Ʈ ĸ͸ Ѵ.
    EventName:=EventName+'('+WriteMethodData(GetTypeData(PropertyEditor.GetPropType))+');';
    // ̺Ʈ ڵ鷯 ۼ ʾ ̺Ʈ ڵ鷯 ۼϰ, ڵ ⸦ ȰȭѴ.
  ////  if  StringGrid2.Cells[1,StringGrid2.Row]='' then   EditorForm.WriteEventCode(EventName)
  ////  else EditorForm.MoveToString(EventName);
  ////  StringGrid2.Cells[1,StringGrid2.Row]:=EventHandler;
end;

{ Get a value from the list of string values. }
procedure TObjectInspector.GetValueProc(const Value: string);
begin
   cbValue.Items.Add(Value);
end;

procedure TObjectInspector.PropertyBoxDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  sr,dr: TRect;
  PropName,PropValue:string;
begin
  with (Control as TListBox).Canvas do
  begin
    Brush.Color := PropertyBox.Color;          {Won't use Windows Selected color.}
    fillRect(Rect);

    // μ ׸
    if (odSelected in State) then Pen.Color := clBtnHighLight else {for 3D look}
      Pen.Color := clBtnShadow;
    Pen.Mode := pmCopy;
    Pen.Width := 1;
    Pen.Style := psSolid;
    MoveTo(Rect.Left, Rect.Bottom-1);
    LineTo(Rect.Right, Rect.Bottom-1);

    // Item ߰ 巡׸   ׸
    if (odSelected in State) then begin
      Pen.Color := clBtnHighLight;
      MoveTo(HalfWay + 2, Rect.Top+1);
      LineTo(Halfway + 2, Rect.Bottom);

      Pen.Color := clBtnShadow;
      MoveTo(Halfway+1, Rect.Top+2);
      LineTo(Halfway+1, Rect.Bottom);

      MoveTo(Rect.Right, Rect.Top);
      LineTo(Rect.Left, Rect.Top);
      LineTo(Rect.Left, Rect.Bottom);

      Pen.Color := clBlack;
      MoveTo(Rect.Right-1, Rect.Top+1);
      LineTo(Rect.Left+1, Rect.Top+1);
      LineTo(Rect.Left+1, Rect.Bottom-1);
    end
    else begin
      Pen.Color := clBtnShadow;
      MoveTo(Halfway, Rect.Top+1);
      LineTo(Halfway, Rect.Bottom);

      Pen.Color := clBtnHighLight;
      MoveTo(HalfWay + 1, Rect.Top+1);
      LineTo(Halfway + 1, Rect.Bottom);
    end;

    SR.Left := Rect.Left+2 ;  SR.Right := HalfWay;
    SR.Top := Rect.Top+1;  SR.Bottom := Rect.Bottom-1;

    if (odSelected in State) then begin
      Inc(SR.Left);
      Inc(SR.Top);
      Inc(SR.Right);
    end;
    // Item Ӽ Ӽ 
    Font.Color := clBlack;
    setbkMode(PropertyBox.Canvas.Handle, Transparent);
    PropName:= Copy(PropertyBox.Items[Index],1,20);
    PropValue:= Copy(PropertyBox.Items[Index],21,255);
    if PropName[1] = '+' then
       TextRect(SR, Sr.Left, SR.Top, PropName )
    else
       TextRect(SR, Sr.Left+10, SR.Top, PropName );
    Font.Color := clNavy;
    TextOut(HalfWay + 4, SR.Top-2, PropValue );

    DR.Left := Rect.Left+2;  DR.Right := Rect.Left+13+2;  {DestRect}
    DR.Top := Rect.Top+1; DR.Bottom := Rect.Top+13+1;

    if (odSelected in State) then begin       { To give 3D look to inspector.}
      Inc(DR.Left); Inc(DR.Right);
      Inc(DR.Top); Inc(DR.Bottom);
      if cbValue.Visible then begin
      // PropertyBox ũѽ cbValue ġ Ѵ.
        with Rect do begin
           if (Top <= 0) and (PropertyBox.ItemIndex <> 0) then
              cbValue.Visible:= False
           else if (Bottom > PropertyBox.Height) then
              cbValue.Visible:= False
           else cbValue.SetBounds(HalfWay+4, Top+2, Right-HalfWay, Bottom-Top);
        end;
      end;
    end;
  end;
  if (not edProperty.Focused) and (odselected in state) then begin
      edProperty.Visible:= False;
      with PropertyBox.Canvas do begin
         EditRect := PropertyBox.ItemRect(PropertyBox.ItemIndex);
         EditRect.Left := HalfWay + 4;
         Brush.Color := clWhite;
         Inc(EditRect.Top,3);
         FillRect(EditRect);
         setbkColor(PropertyBox.Canvas.Handle, GetsysColor(color_HighLight));
         setbkMode(PropertyBox.Canvas.Handle, Opaque);
         Font.Color := clhighlightText;
         Canvas.Brush.Color := clHighlight;
         TextOut(EditRect.Left, EditRect.Top, PropValue);
         DestroyCaret;
         Createcaret(PropertyBox.Handle, 0, 1, (EditRect.Bottom - Editrect.Top) - 2);
         SetCaretPos(EditRect.Left + TextWidth(PropValue), EditRect.Top+1);
         ShowCaret(PropertyBox.Handle);
      end;
  end
 // if (odFocused in State) and (odselected in state) then
 //     (Control as TListbox).Canvas.DrawFocusRect(rect);
end;

procedure TObjectInspector.PropertyBoxMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  R,sr : TRect;
begin
  if Abs(X - Halfway) <= 3 then  SplitDrag := True
  else
  if (X >= EditRect.left) and (X <= EditRect.Right) and
     (y >= EditRect.Top) and (Y <= EditRect.Bottom) then begin
     PropertyBox.Cursor := crDefault;
    // SetEditor(PropertyList.Objects[PropertyBox.ItemIndex] as TPropertyEditor)
  end;
end;

procedure TObjectInspector.PropertyBoxMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
var
  TempR : TRect;
  LB : TListBox;
begin
  if PageControl1.ActivePage = TabProperty then  Lb := PropertyBox
  else lb := EventBox;
  if SplitDrag then begin
    if X > Lb.ClientWidth - 20 then X := Lb.ClientWidth - 20;
    if X < 20 then X := 20;

    if Halfway < X then TempR:= Rect(Halfway, 0, lb.ClientWidth, lb.ClientHeight)
    else  TempR:= Rect(X, 0, lb.ClientWidth, lb.ClientHeight);
    HalfWay := X;

    InvalidateRect(lb.Handle, @TempR, False);
    InvalidateRect(lb.Handle, @EditRect, False);
    PostMessage(lb.Handle, WM_Paint, 0, 0);
    Exit;
  end;
  if Abs(x - Halfway) <= 3 then   lb.Cursor := crHSplit
  else
  if (X >= EditRect.left) and (X <= EditRect.Right) and
     (Y >= EditRect.Top) and (Y <= EditRect.Bottom) then
       lb.Cursor := crIBeam else lb.Cursor := crDefault;
end;

procedure TObjectInspector.PropertyBoxMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
   SplitDrag := False;
end;

procedure TObjectInspector.FormShow(Sender: TObject);
begin
  Halfway := PropertyBox.Width div 2;
end;

procedure TObjectInspector.EventBoxDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  SR,DR: TRect;
  EventName,EventValue:string;
begin
  with (Control as TListBox).Canvas do
  begin
    Brush.Color := EventBox.Color;          {Won't use Windows Selected color.}
    FillRect(Rect);
    // μ ׸
    if (odSelected in State) then Pen.Color := clBtnHighLight else {for 3D look}
      Pen.Color := clBtnShadow;
    Pen.Mode := pmCopy;
    Pen.Width := 1;
    Pen.Style := psSolid;
    MoveTo(rect.Left, Rect.Bottom-1);
    LineTo(rect.Right, Rect.Bottom-1);

    // Item ߰ 巡׸   ׸
    if (odSelected in State) then begin
      Pen.Color := clBtnHighLight;
      MoveTo(HalfWay + 2, Rect.Top+1);
      LineTo(Halfway + 2, Rect.Bottom);

      Pen.Color := clBtnShadow;
      MoveTo(Halfway+1, rect.Top+2);
      LineTo(Halfway+1, rect.Bottom);

      MoveTo(Rect.Right, Rect.Top);
      LineTo(Rect.Left, Rect.Top);
      LineTo(Rect.Left, Rect.Bottom);

      Pen.Color := clBlack;
      MoveTo(Rect.Right-1, Rect.Top+1);
      LineTo(Rect.Left+1, Rect.Top+1);
      LineTo(Rect.Left+1, Rect.Bottom-1);
    end
    else begin
      Pen.Color := clBtnShadow;
      MoveTo(Halfway, rect.Top+1);
      LineTo(Halfway, rect.Bottom);

      Pen.Color := clBtnHighLight;
      MoveTo(HalfWay + 1, Rect.Top+1);
      LineTo(Halfway + 1, Rect.Bottom);
    end;

    SR.Left := Rect.Left+2 ;  SR.Right := HalfWay;
    SR.Top := Rect.Top+1;  SR.Bottom := Rect.Bottom-1;

    if (odSelected in State) then begin
      Inc(SR.Left);
      Inc(SR.Top);
      Inc(SR.Right);
    end;
    // Item ̺Ʈ ̺Ʈ 
    Font.Color := clBlack;
    setbkMode(EventBox.Canvas.Handle, Transparent);
    EventName:= Copy(EventBox.Items[Index],1,20);
    EventValue:= Copy(EventBox.Items[Index],21,255);
    TextRect(SR, Sr.Left + 10, SR.Top, EventName);
    Font.Color := clNavy;
    TextOut(HalfWay + 4, SR.Top, EventValue );

    DR.Left := Rect.Left+2;  DR.Right := Rect.Left+13+2;  {DestRect}
    DR.Top := Rect.Top+1; DR.Bottom := Rect.Top+13+1;

    if (odSelected in State) then begin       { To give 3D look to inspector.}
      Inc(DR.Left); Inc(DR.Right);
      Inc(DR.Top); Inc(DR.Bottom);
    end;
  end;

  if (not edEvent.Focused) and (odselected in state) then
      with EventBox.Canvas do begin
         EditRect := EventBox.ItemRect(EventBox.ItemIndex);
         EditRect.Left := HalfWay + 2;
         Brush.Color := clWhite;
         Inc(EditRect.Top,4);
         Fillrect(EditRect);
         setbkColor(EventBox.Canvas.Handle, GetsysColor(color_HighLight));
         setbkMode(EventBox.Canvas.Handle, Opaque);
         Font.Color := clhighlightText;
         Canvas.Brush.Color := clHighlight;
         TextOut(EditRect.Left, EditRect.Top, EventValue);
         DestroyCaret;
         Createcaret(EventBox.Handle, 0, 1, (EditRect.Bottom - Editrect.top) - 2);
         SetCaretPos(EditRect.Left + TextWidth(EventValue), EditRect.Top+1);
         ShowCaret(EventBox.handle);
      end;
  if (odFocused in State) and (odselected in state) then
      (Control as TListbox).Canvas.DrawFocusRect(Rect);
end;

procedure TObjectInspector.EventBoxMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  P : TPoint;
begin
  if Abs(X - Halfway) <= 3 then  SplitDrag := True
  else
  if (X >= EditRect.Left) and (X <= EditRect.Right) and
     (y >= EditRect.Top) and (Y <= EditRect.Bottom) then begin
     P:= Point(X,Y);
     EventBox.Cursor := crDefault;
   //  ShowEventInput;
     MapWindowPoints(EventBox.Handle, edEvent.Handle, P, 1);
     PostMessage(edEvent.Handle, WM_LButtonDown, mk_lbutton, P.Y shl 16 + P.X);
  end;
end;

procedure TObjectInspector.EventBoxMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
var
  TempR : TRect;
  LB : TListBox;
begin
  if PageControl1.ActivePage = TabProperty then  Lb := PropertyBox
  else lb := EventBox;
  if SplitDrag then begin
    if X > Lb.ClientWidth - 20 then X := Lb.ClientWidth - 20;
    if X < 20 then X := 20;

    if Halfway < X then TempR:= Rect(Halfway, 0, lb.ClientWidth, lb.ClientHeight)
    else  TempR:= Rect(X, 0, lb.ClientWidth, lb.ClientHeight);
    HalfWay := X;

    InvalidateRect(lb.Handle, @TempR, False);
    InvalidateRect(lb.Handle, @EditRect, False);
    PostMessage(lb.Handle, WM_Paint, 0, 0);
    Exit;
  end;
  if Abs(x - Halfway) <= 3 then   lb.Cursor := crHSplit
  else
  if (X >= EditRect.left) and (X <= EditRect.Right) and
     (Y >= EditRect.Top) and (Y <= EditRect.Bottom) then
       lb.Cursor := crIBeam else lb.Cursor := crDefault;
end;

procedure TObjectInspector.EventBoxMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
   if SplitDrag then begin   // 巡װ  Ʈ ġ 
      edEvent.Left := EditRect.left;
      edEvent.Top := EditRect.Top;
      edEvent.Width := EditRect.Right - EditRect.Left;
      edEvent.Height := EditRect.Bottom - EditRect.Top;
   end;
   SplitDrag := False;
end;

// ListBox ItemIndex Ӽ ش.
function TObjectInspector.GetPropertyName: string ;
begin
   Result:='';
   if PropertyBox.ItemIndex = -1 then Exit;
   Result:= Trim(Copy(PropertyBox.Items[PropertyBox.ItemIndex],1,20));
end;

// ListBox ItemIndex ̺Ʈ ش.
function TObjectInspector.GetEventName: string ;
begin
   Result:='';
   if EventBox.ItemIndex = -1 then Exit;
   Result:= Trim(Copy(EventBox.Items[EventBox.ItemIndex],1,20));
end;

// ListBox ItemIndex Ӽ ش.
function TObjectInspector.GetPropertyValueName: string ;
begin
   Result:='';
   if PropertyBox.ItemIndex = -1 then Exit;
   Result:= Trim(Copy(PropertyBox.Items[PropertyBox.ItemIndex],21,255));
end;

// ListBox ItemIndex ̺Ʈ ش.
function TObjectInspector.GetEventValueName: string ;
begin
   Result:='';
   if EventBox.ItemIndex = -1 then Exit;
   Result:= Trim(Copy(EventBox.Items[EventBox.ItemIndex],21,255));
end;

// ListBox ItemIndex Ӽ Ѵ.
procedure TObjectInspector.SetPropertyValueName(Value:string) ;
begin
   if PropertyBox.ItemIndex = -1 then Exit;
   with PropertyBox do
      Items[ItemIndex]:= Copy(Items[ItemIndex],1,20)+ Value;
end;

// Ŭ Ӽ Ӽ ⸦ Ѵ.
procedure TObjectInspector.PropertyBoxClick(Sender: TObject);
begin
   with PropertyList do
      if PropertyList.Count>0 then
         SetEditor(Objects[PropertyBox.ItemIndex] as TPropertyEditor)
      else SetEditor(nil);
end;

procedure TObjectInspector.PropertyBoxKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if key = vk_return then begin
    Key := 0;
    exit;
  end;
  if not (Key in [vk_up, vk_down, vk_prior, vk_next]) then begin
    edProperty.left := EditRect.left;
    edProperty.Top := EditRect.Top;
    edProperty.Width := EditRect.Right - EditRect.Left;
    edProperty.Height := EditRect.Bottom - EditRect.Top;
    edProperty.Visible := True;
    edProperty.SetFocus;
    PostMessage(edProperty.Handle, WM_KeyDown, Key, 0);
    Key := 0;
  end;
end;

procedure TObjectInspector.EventBoxClick(Sender: TObject);
begin
   with EventList do
      if Count>0 then
         SetEditor(Objects[EventBox.ItemIndex] as TPropertyEditor)
      else  SetEditor(nil);
end;

// Ӽ  Ŭϴ ,  ʷ ٲٰ, Ӽ Ѵ.
// ListBox Style Ӽ csDropDown Ǿ ִ 쿡 
procedure TObjectInspector.cbValueDblClick(Sender: TObject);
begin
   with cbValue do begin
      if ItemIndex<0 then ItemIndex:=0
      else if ItemIndex=(Items.Count-1) then ItemIndex:=0
      else ItemIndex:=ItemIndex+1;
      cbValueChange(self);
   end;
end;

procedure TObjectInspector.BrowseClick(Sender: TObject);
begin
   EditorProperty;
end;

procedure TObjectInspector.edPropertyKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
   case Key of
     VK_DOWN:
        if PropertyBox.ItemIndex< (PropertyBox.Items.Count-1) then begin
           if edProperty.Modified then begin //Edit  Ǿ
              PropSetValue(edProperty.Text);
              with PropertyBox do
                 Items[ItemIndex]:= Format('%-20s',[GetPropertyName]) + edProperty.Text;
           end;
        end;
     VK_UP:
        if PropertyBox.ItemIndex >= 1 then begin
           if edProperty.Modified then begin //Edit  Ǿ
              PropSetValue(edProperty.Text);
              with PropertyBox do
                 Items[ItemIndex]:= Format('%-20s',[GetPropertyName]) + edProperty.Text;
           end;
        end;
   end;
   if (Key in [vk_up, vk_down, vk_prior, vk_next]) then begin
      PropertyBox.SetFocus;
      PostMessage(PropertyBox.Handle, WM_KeyDown, Key, 0);
      Key := 0;
   end;
end;

procedure TObjectInspector.edPropertyKeyPress(Sender: TObject;
  var Key: Char);
begin
   if Key=#13 then begin
      PropertyBox.SetFocus;
      PropSetValue(edProperty.Text);
      with PropertyBox do
         Items[ItemIndex]:= Format('%-20s',[GetPropertyName]) + edProperty.Text;
   end;
end;

procedure TObjectInspector.cbValueChange(Sender: TObject);
begin
   PropSetValue(cbValue.Text);
   with PropertyBox do
       Items[ItemIndex]:= Format('%-20s',[GetPropertyName]) + cbValue.Text;
end;

procedure TObjectInspector.cbValueKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
   if Key = vk_Return then Begin
      PropertyBox.SetFocus;
      Key := 0;
      Exit;
   end;
   if (Key in [vk_up, vk_down, vk_prior, vk_next]) then begin
      PropertyBox.Setfocus;
      PostMessage(PropertyBox.Handle, WM_KeyDown, Key, 0);
      Key := 0;
   end;
end;

procedure TObjectInspector.cbValueDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
begin
  with (Control as TComboBox).Canvas do
  begin
    FillRect(Rect);
   // Font.Color := clNavy;
    TextOut(Rect.Left, Rect.Top, cbValue.Items[Index]);
  end;
end;

procedure TObjectInspector.FormActivate(Sender: TObject);
begin
  ComponentList.ItemIndex:=0;
end;

// ο  ۼϰų  ̸ ȯ  Object Inspector 
//  ° ٽ д´.
procedure TObjectInspector.InitializeInspector;
var
  I:Integer;
begin
   ComponentList.Clear;
   ComponentList.Items.Add(ProxyDesigner.Form.Name+':TForm');
   with ProxyDesigner.Form do  begin
    for I:=0 to  ComponentCount-1 do
        if (Components[I].ClassType<>TGrabHandle) and (Components[I].ClassType<>TWrapperControl) then
           ComponentList.Items.Add(Components[I].Name+':'+Components[I].ClassName)
    end;
    ComponentList.ItemIndex:=0;
end;

// ۳Ʈ Ͽ ۳Ʈ ϴ , ش ۳Ʈ ϰ Ӽ ش.
procedure TObjectInspector.ComponentListClick(Sender: TObject);
var
  Component:TComponent;
  Wrapper:TComponent;
begin
   Component := GetComponent(Wrapper);
   DisplayProperty(Wrapper,Component);
end;

procedure TObjectInspector.edEventDblClick(Sender: TObject);
var
   EventName: string;
   EventHandler:string;
begin
    EventName:=ExtractComponentName(ComponentList.text);
    with EventBox do
       EventName:= Trim(EventName+Copy(Items[ItemIndex],3,20));
    EventHandler:= EventName;
    // ̺Ʈ ĸ͸ Ѵ.
    EventName:=EventName+'('+WriteMethodData(GetTypeData(PropertyEditor.GetPropType))+');';
    // ̺Ʈ ڵ鷯 ۼ ʾ ̺Ʈ ڵ鷯 ۼϰ, ڵ ⸦ ȰȭѴ.
    if  edEvent.Text = '' then   EditorForm.WriteEventCode(EventName)
    else EditorForm.MoveToString(EventName);
    with EventBox do
       Items[ItemIndex]:=
          Format('%20s%s',[Copy(Items[ItemIndex],1,20),EventHandler]);
end;

procedure TObjectInspector.PropertyBoxDblClick(Sender: TObject);
var
  Po:TPoint;
  PropInfo:PPropInfo;
  PropName:string;
  Editor:TPropertyEditor;
begin
   GetCursorPos(Po);
   Po:= PropertyBox.ScreenToClient(Po);
   if Po.x < HalfWay then begin
      if PropertyBox.Items[PropertyBox.ItemIndex][1] = '+' then  begin
         PropName:= Trim(Copy(PropertyBox.Items[PropertyBox.ItemIndex],2,20));
         PropertyBox.Items[PropertyBox.ItemIndex]:= '-' + PropName;
         //PropInfo:= GetPropInfo(SelectComponent.ClassInfo, PropName);
         //InsertPropInfo(PropInfo, SelectComponent);
         Editor:= TPropertyEditor(PropertyList.Objects[PropertyBox.ItemIndex]);
         if Editor <> nil then
            Editor.GetProperties(GetSubPropEditor);
      end
      else if PropertyBox.Items[PropertyBox.ItemIndex][1] = '-' then  begin
         PropName:= Trim(Copy(PropertyBox.Items[PropertyBox.ItemIndex],2,20));
         PropertyBox.Items[PropertyBox.ItemIndex]:= '+' + PropName;
         PropInfo:= GetPropInfo(SelectComponent.ClassInfo, PropName);
         DeletePropInfo(PropInfo, SelectComponent);
      end;
   end;
end;

//cbValue ItemIndex Text  ε Ѵ.
// cbValue Style csDropDownList ߱ 
// cbValue.Text:= Text  ҿ  ̴.
procedure TObjectInspector.SetcbValueIndex(Text:string);
var
  Index:Integer;
begin
   Index:= cbValue.Items.IndexOf(Text);
   if Index<>-1 then cbValue.ItemIndex:= Index
   else cbValue.Text:= GetPropertyValueName;
end;

end.
